package com.jtauber.fop.fo.pagination;

import com.jtauber.fop.fo.*;
import com.jtauber.fop.fo.properties.*;

import com.jtauber.fop.layout.Document;
import com.jtauber.fop.layout.PageMaster;
import com.jtauber.fop.layout.Region;

import com.jtauber.fop.apps.FOPException;				   

public class SimplePageMaster extends FObj {
	
	public static class Maker extends FObj.Maker {
		public FObj make(Document doc, FObj parent, PropertyList propertyList) throws FOPException {
			return new SimplePageMaster(doc, parent, propertyList);
		}
	}

	RegionBody regionBody;
	RegionBefore regionBefore;
	RegionAfter regionAfter;
	
	LayoutMasterSet layoutMasterSet;
	PageMaster pageMaster;
		
	protected SimplePageMaster(Document doc, FObj parent, PropertyList propertyList) throws FOPException {
		super(doc, parent, propertyList);
		this.foName = "fo:simple-page-master";

		if (parent.foName.equals("fo:layout-master-set")) {
			this.layoutMasterSet = (LayoutMasterSet)parent;
			String pm = ((PageMasterName)this.properties.get("page-master-name")).string();
			if (pm == null) {
				System.err.println("WARNING: simple-page-master does not have a page-master-name and so is being ignored");
			} else {
				this.layoutMasterSet.addLayoutMaster(pm,this);
			}
		} else {
			throw new FOPException("fo:simple-page-master must be child of fo:layout-master-set, not "+parent.foName);
		}
	}
	protected void addCharacters(char data[], int start, int length) {}
	protected void end(Document doc) {
		int pageWidth = ((PageWidth)this.properties.get("page-width")).length().value();
		int pageHeight = ((PageHeight)this.properties.get("page-height")).length().value();

		int marginTop = ((MarginTop)this.properties.get("margin-top")).length().value();
		int marginBottom = ((MarginBottom)this.properties.get("margin-bottom")).length().value();
		int marginLeft = ((MarginLeft)this.properties.get("margin-left")).length().value();
		int marginRight = ((MarginRight)this.properties.get("margin-right")).length().value();

		int cont_rect_sx = marginLeft;
		int cont_rect_sy = pageHeight - marginTop;
		int cont_rect_w = pageWidth - marginLeft - marginRight;
		int cont_rect_h = pageHeight - marginTop - marginBottom;
		
		this.pageMaster = new PageMaster(pageWidth, pageHeight);
		this.pageMaster.addBody(this.regionBody.makeRegion(cont_rect_sx,cont_rect_sy,cont_rect_w,cont_rect_h));
		
		if (this.regionBefore != null)
			this.pageMaster.addBefore(this.regionBefore.makeRegion(cont_rect_sx,cont_rect_sy,cont_rect_w,cont_rect_h));
		if (this.regionAfter != null)
			this.pageMaster.addAfter(this.regionAfter.makeRegion(cont_rect_sx,cont_rect_sy,cont_rect_w,cont_rect_h));
	}
	protected void finishText() {}
	PageMaster getPageMaster() {
		return this.pageMaster;
	}
/**
 * This method was created in VisualAge.
 * @return FObjMaker
 */
public static FObj.Maker maker() {
	return new SimplePageMaster.Maker();
}
	protected void setRegionAfter(RegionAfter region) {
		this.regionAfter = region;
	}
	protected void setRegionBefore(RegionBefore region) {
		this.regionBefore = region;
	}
	protected void setRegionBody(RegionBody region) {
		this.regionBody = region;
	}
}