package com.jtauber.fop.layout;

import java.io.PrintWriter;
import java.util.Vector;
import java.io.IOException;
import java.util.Enumeration;
import com.jtauber.fop.fo.flow.StaticContent;

import com.jtauber.fop.apps.FOPException;				   

public class Document {

	Layout layout; // the output-dependent layout code interface
	FontInfo fontInfo; // the object that provides output-independent info on fonts
	
	FontState defaultFontState;

	PageMasterFactory pageMasterFactory;

	Vector pageList; // list of all the pages
	Page currentPage;
	Area currentArea;

	StaticContent staticBefore;
	StaticContent staticAfter;

	private int currentPageNumber;
	private boolean currentAreaContainerIsStatic;
	
	public Document(Layout layout) {

		this.layout = layout;
		this.fontInfo = new FontInfo();
		layout.setupFontInfo(this.fontInfo);
		
		this.currentPageNumber = 1; // this shouldn't be here
		this.pageList = new Vector();
	}
	public void addPage(Page p) {
		pageList.addElement(p);
	}
	public void addText(FontState fs, char[] ca, int ts, int te) throws FOPException {
		int start = ((BlockArea)currentArea).addText(fs,ca,ts,te);
		if (start != -1) {
			BlockArea ba = (BlockArea)currentArea;
			endBlockArea(0);
			if (this.currentAreaContainerIsStatic) {
				System.err.println("WARNING: static-content was truncated");
			} else {
				endPage();
				startPage();
				startBlockAreaDuplicate(ba); // starts a new currentBlockArea but with same properties as previous one
				addText(fs,ca,start,te);
			}
		}
	}
	public void addText(FontState fs, String text) throws FOPException {
		char[] ca = text.toCharArray();
		int ts = 0;
		int te = text.length();
		this.addText(fs,ca,ts,te);
	}
	public void endBlockArea(int space_after) {
		currentArea.end();
		if (space_after != 0) {
			currentArea.addDisplaySpace(space_after);
		}
		currentArea.parent.increaseHeight(currentArea.getHeight());
		currentArea = currentArea.parent;
		if (currentArea instanceof BlockArea)
			currentArea.start();
	}
	public void endPage() {
			this.addPage(currentPage);
			this.currentPageNumber++;
	}
	public FontInfo getFontInfo() {
		return this.fontInfo;
	}
	public String getPageNumber() {
		return Integer.toString(this.currentPageNumber);
	}
public Vector getPages() {
	return this.pageList;
}
	public void setDefaultFontState(FontState defaultFontState) {
		this.defaultFontState = defaultFontState;
	}
	public void setPageMasterFactory(PageMasterFactory pageMasterFactory) {
		this.pageMasterFactory = pageMasterFactory;
	}
	public void setStaticAfter(StaticContent sc) {
		this.staticAfter = sc;
	}
	public void setStaticBefore(StaticContent sc) {
		this.staticBefore = sc;
	}
	public void startBlockArea(FontState fontState, int align, int alignLastLine, int lineHeight, int startIndent, int endIndent, int spaceBefore) {
		if (currentArea instanceof BlockArea)
			currentArea.end();
		if (spaceBefore != 0)
			currentArea.addDisplaySpace(spaceBefore);

		BlockArea ba = new BlockArea(fontState,	currentArea.getWidth(), currentArea.spaceLeft(), startIndent, endIndent, align, alignLastLine, lineHeight);
		ba.start();
		currentArea.addChild(ba);
		currentArea = ba;
	}
	public void startBlockAreaDuplicate(BlockArea ba) {
		startBlockArea(ba.fontState,ba.align,ba.alignLastLine,ba.lineHeight,ba.startIndent,ba.endIndent,0);
	}
	public void startPage() throws FOPException {

		// GET PAGE MASTER
		PageMaster pageMaster = this.pageMasterFactory.getNextPageMaster();
		while (pageMaster == null) {
			// System.err.println("moving on to next sequence specifier");
			this.pageMasterFactory = this.pageMasterFactory.getNext();
			if (this.pageMasterFactory == null) {
				throw new FOPException("out of sequence specifiers (subsequent versions of FOP will allow this)");
			}
			pageMaster = this.pageMasterFactory.getNextPageMaster();
		}

		// MAKE A NEW PAGE
		
		currentPage = pageMaster.makePage();
		
		if (this.staticBefore != null) {
			if (currentPage.getBefore() != null) {
				this.currentAreaContainerIsStatic = true;
				currentArea = currentPage.getBefore();
				this.staticBefore.layout();
			}
		}
		
		if (this.staticAfter != null) {
			if (currentPage.getAfter() != null) {
				this.currentAreaContainerIsStatic = true;
				currentArea = currentPage.getAfter();
				this.staticAfter.layout();
			}
		}

		this.currentAreaContainerIsStatic = false;
		currentArea = currentPage.getBody();
	}
	public void toXML(PrintWriter writer) throws IOException {
		Enumeration e = pageList.elements();
		writer.println("<Document>"); 
		while (e.hasMoreElements()) {
			((Page)e.nextElement()).toXML(writer);
		}
		writer.println("</Document>");
		writer.flush();
	}
}