package com.jtauber.fop.layout;

import java.util.Vector;
import java.util.Enumeration;

public class LineArea extends Area {
	
	protected int lineHeight;
	protected int halfLeading;
	
	private int finalWidth; // in millipoints
	private int currentWidth; // in millipoints
	private int prev;
	String text  = "";
	public LineArea(FontState fontState, int lineHeight, int halfLeading, int allocationWidth, int startIndent, int endIndent) {
		super(fontState);		

		this.lineHeight = lineHeight;
		this.halfLeading = halfLeading;
		
		this.contentRectangleWidth = allocationWidth - startIndent - endIndent;
		this.fontState = fontState;

		if (startIndent !=0)
			addChild(new InlineSpace(1000 * startIndent));

		finalWidth = 0;
		currentWidth = 0;
		prev = 0;
	}
	private void addChildAtStart(Box child) {
		this.children.insertElementAt(child,0);
	}
	public int addText(char data[], int start, int end) {
		int WHITESPACE = 1;
		int TEXT = 2;

		int wordStart = start;
		int wordLength = 0;
		
		int spaceWidth = fontState.width(32);
		
		for (int i=start; i<end; i++) {
			int charWidth = fontState.width(data[i]);
			if ((data[i] == ' ') ||
				(data[i] == '\n') ||
				(data[i] == '\r') ||
				(data[i] == '\t')) { // whitespace
					if (prev == WHITESPACE) {
							// do nothing
					} else if (prev == TEXT) {
						String s = new String(data, wordStart, wordLength);
						addChild(new InlineArea(fontState, s));
						this.text += s;
						prev = WHITESPACE;
					} else {
						prev = WHITESPACE;
					}
			} else {
				//System.err.println(currentWidth + " " + data[i] + " " + charWidth + " " + (currentWidth + charWidth) + " [" + finalWidth + "]");
				if (prev == WHITESPACE) {
					finalWidth = currentWidth; // - spaceWidth;
					if ((currentWidth + spaceWidth) > (1000 * this.contentRectangleWidth)) {
						// addChild(new InlineArea(currentFontState, "[1-" + actualWidth + "]"));
						return i;
					}
					addChild(new InlineSpace(spaceWidth));
					this.text += " ";
					currentWidth += spaceWidth;
					prev = TEXT;
					wordStart = i;
					wordLength = 1;
				} else if (prev == TEXT) {
					wordLength++;
				} else {
					prev = TEXT;
					wordStart = i;
					wordLength = 1;
				}
				if ((currentWidth + charWidth) > (1000 * this.contentRectangleWidth)) {
					currentWidth -= spaceWidth;
					// addChild(new InlineArea(currentFontState, "[2-" + actualWidth + "]"));
					return wordStart;
				}
				currentWidth += charWidth;
			}
		}
/*		if (prev == WHITESPACE) {
			addChild(new InlineSpace(spaceWidth));
			currentWidth += spaceWidth;
		} else */ if (prev == TEXT) {
					  String s = new String(data, wordStart, wordLength);
					  this.text += s;
			addChild(new InlineArea(fontState, s));
		}
		finalWidth = currentWidth;
		return -1;
	}
	public void align(int type) {
		int padding;
			
		switch (type) {
		case 1: { // left
					// do nothing
					break;
				}
		case 2: { // right
					addChildAtStart(new InlineSpace((1000 * this.contentRectangleWidth) - finalWidth));
					break;
				}
		case 3: { // center
					addChildAtStart(new InlineSpace(((1000 * this.contentRectangleWidth) - finalWidth) / 2));
					break;
				}
		case 4: { // justify
					Vector spaceList = new Vector();
					int spaceCount = 0;
					Enumeration e = children.elements();
					while (e.hasMoreElements()) {
						Box b = (Box)e.nextElement();
						if (b.getClass().getName().equals("com.jtauber.fop.layout.InlineSpace")) {
							InlineSpace space = (InlineSpace)b;
							spaceList.addElement(space);
							spaceCount++;
						}
					}
					if (spaceCount > 0) {
						padding = ((1000 * this.contentRectangleWidth) - finalWidth) / spaceCount;
					} else { // no spaces
						System.err.println("WARNING: trying to justify a line with no spaces. This has been know to hang current version of FOP. I'm looking into it.");
						padding = 0;
					}
					Enumeration f = spaceList.elements();
					while (f.hasMoreElements()) {
						InlineSpace space2 = (InlineSpace)f.nextElement();
						int i = space2.getSize();
						space2.setSize(i + padding);
					}
				}
		}
	}
	public void changeFont(FontState fontState) {
		this.fontState = fontState;
	}
	public int getHeight() {
		return this.lineHeight;
	}
	 void toXML(java.io.PrintWriter writer) {
		writer.print("<LineArea>");
		java.util.Enumeration e = children.elements();
		while (e.hasMoreElements()) {
			((Box)e.nextElement()).toXML(writer);
		}
		writer.println("</LineArea>");
	}
}