package com.jtauber.fop.layout.pdf;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

import com.jtauber.fop.layout.*;
import com.jtauber.pdf.*;

public class PDFLayout implements Layout {
	
	PDFDocument pdfDoc;
	PDFResources pdfResources;

	PDFStream currentStream;

	String currentFontName;
	int currentFontSize;
	int currentOffset = 0;
	int initialOffset = 0;
	boolean inText = false;
	public PDFLayout(String producer) {
		pdfDoc = new PDFDocument(producer);
	}
public void go(Document doc, PrintWriter writer) throws IOException {
	System.err.println("converting areas/spaces to PDF");
	this.pdfResources = this.pdfDoc.getResources();
	Enumeration e = doc.getPages().elements();
	while (e.hasMoreElements()) {
		this.makePage((Page)e.nextElement());
	}
	System.err.println("writing out PDF");
	this.pdfDoc.output(writer);
}
	void makeAreaContainer(AreaContainer areaContainer) {
		currentStream.add(areaContainer.x_start + " " + (areaContainer.y_start - areaContainer.margin_top) + " Td\n");

		Enumeration e = areaContainer.getChildren().elements();
		while (e.hasMoreElements()) {
			Box b = (Box)e.nextElement();
			if (b instanceof BlockArea)
				makeBlockArea((BlockArea)b);
			else if (b instanceof DisplaySpace)
				makeDisplaySpace((DisplaySpace)b);
			else
				System.err.println("Area container had something other than BlockArea and DisplaySpace");
		}
	}
	public void makeBlockArea(BlockArea blockArea) {
		String name = blockArea.getFontState().getFontName();
		int size = blockArea.getFontState().getFontSize();

		if ((!name.equals(this.currentFontName)) || (size != this.currentFontSize)) {
			this.currentFontName = name;
			this.currentFontSize = size;
			currentStream.add("/" + name + " " +  size + " Tf\n");
		}
		
		Enumeration e = blockArea.getChildren().elements();
		while (e.hasMoreElements()) {
			Box b = (Box)e.nextElement();
			if (b instanceof LineArea)
				makeLineArea((LineArea)b);
			else if (b instanceof DisplaySpace)
				makeDisplaySpace((DisplaySpace)b);
			else
				System.err.println("Block area had something other than LineArea and DisplaySpace");
		}
	}
	public void makeDisplaySpace(DisplaySpace displaySpace) {
		currentStream.add("0 " + (-1 * displaySpace.getSize()) + " Td\n");
	}
	public void makeInlineArea(InlineArea inlineArea) {
		// FIXME this function badly needs optimization
		
		String pdf = "";
		
		String name = inlineArea.getFontState().getFontName();
		int size = inlineArea.getFontState().getFontSize();

		if ((!name.equals(this.currentFontName)) || (size != this.currentFontSize)) {
			this.currentFontName = name;
			this.currentFontSize = size;
			if (inText) {
				pdf += " ] TJ\n";
				inText = false;
			}
			pdf += "/" + name + " " + size + " Tf\n";
		}

		if (!inText) {
			pdf += "[";
			inText = true;
			initialOffset += currentOffset;
			currentOffset = initialOffset;
		}
		if (currentOffset != 0)
			pdf += " " + currentOffset;

		// this is dumb, what should I do instead?
		char[] ca = inlineArea.getText().toCharArray();
		pdf += " (";
		for (int i=0; i < ca.length; i++) {
			if (ca[i] == '(')
				pdf += "\\(";
			else if (ca[i] == ')')
				pdf += "\\)";
			else
				pdf += ca[i];
		}
		pdf += ")";

		currentStream.add(pdf);
		currentOffset = 0;
	}
	public void makeInlineSpace(InlineSpace inlineSpace) {
		currentOffset -= (inlineSpace.getSize()/currentFontSize);
	}
	public void makeLineArea(LineArea lineArea) {
		currentStream.add("0 " + (-1 * lineArea.getFontState().getFontSize()) + " Td\n");
		currentStream.add("/" + lineArea.getFontState().getFontName() + " " +  lineArea.getFontState().getFontSize() + " Tf\n");

		currentOffset = 0;
		initialOffset = 0;
		
		Enumeration e = lineArea.getChildren().elements();
		while (e.hasMoreElements()) {
			Box b = (Box)e.nextElement();
			if (b instanceof InlineArea)
				makeInlineArea((InlineArea)b);
			else if (b instanceof InlineSpace)
				makeInlineSpace((InlineSpace)b);
			else
				System.err.println("Line area had something other than InlineArea and InlineSpace");
		}
		if (inText) {
			currentStream.add(" ] TJ\n");
			inText = false;
		}
	}
	 void makePage(Page page) {
		AreaContainer body, before, after;
		
		currentStream = this.pdfDoc.makeStream();
		body = page.getBody();
		before = page.getBefore();
		after = page.getAfter();
		
		currentStream.add("BT\n");
		makeAreaContainer(body);
		currentStream.add("ET\n");

		if (before != null) {
			currentStream.add("BT\n");
			makeAreaContainer(before);
			currentStream.add("ET\n");
		}

		if (after != null) {
			currentStream.add("BT\n");
			makeAreaContainer(after);
			currentStream.add("ET\n");
		}

		this.pdfDoc.makePage(this.pdfResources,currentStream,page.getWidth(),page.getHeight());
	}
public void setupFontInfo(FontInfo fontInfo) {
	FontSetup.setup(fontInfo);
	FontSetup.addToResources(this.pdfDoc, fontInfo);
}
}