package com.jtauber.pdf;

import java.io.IOException;
import java.io.PrintWriter;

/**
 * PDF Object
 * 
 *  A PDF Document is essentially a collection of these objects. A PDF
 *  Object has a number and a generation (although the generation will always
 *  be 0 in new documents).
 */
public abstract class PDFObject {
	protected int number;
	protected int generation;
	/**
	 * create an empty object with the given number
	 */
	public PDFObject(int number) {
		this.number = number;
		this.generation = 0;
	}
	/**
	 * write the PDF represention of this object to the given filehandle
	 * and return the length of that representation
	 */
	int output(PrintWriter writer) throws IOException {
		String pdf = this.toPDF();
		writer.write(pdf);
		return pdf.length();
	}
	protected String referencePDF() {
		return this.number + " " + this.generation + " R";
	}
	/**
	 * represent object as PDF
	 * 
	 * <number> <generation> obj
	 * <dictionary>
	 * endobj
	 */
	abstract String toPDF();
}